
package zoomarket;

import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;


public class Repository {

    protected LinkedList<Serializable> items;
    private ObjectInputStream input;
    private ObjectOutputStream output;


	public LinkedList<Serializable> getItems(){
		return items;
	}

    public Serializable getItem( int index ) throws ZooException{
        try{
            return items.get( index );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while getting item: Index " + index
                    + " is out of bounds. There are " + size() + " available products" );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while getting item at index " + index );
        }
    }

    public void addItem( Serializable item ){
        if( item == null ) return;
        items.add( item );
    }

    public void insertItem( int dest, Serializable item ) throws ZooException{
        if( item == null ) return;
        try{
            items.add( dest, item );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while inserting item: Index " + dest
                    + " is out of bounds. There are " + size() + " available items" );
        }
    }

    public boolean removeItem( int index ) throws ZooException{
        try{
            return items.remove( index ) != null;
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while removing item: Index " + index
                    + " is out of bounds. There are " + size() + " available items" );
        }
    }

    public void openDataBase( String file ) throws ZooException{
        renewList();
        addDataBase( file );
    }

    public void addDataBase( String file ) throws ZooException{
        openFileForInput( file );

        try{
            while( true ){
                items.add( (Serializable) input.readObject() );
            }
        }
        catch( EOFException ex ){
            // File finished.
            return;
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while reading from file '" + file + "'" );
        }
        finally{
            closeFileForInput();
        }
    }

    public void saveDataBase( String file ) throws ZooException{
        try{
            openFileForOutput(file);

            Iterator iter = items.iterator();
            while( iter.hasNext() ){
                output.writeObject( iter.next() );
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while saving repository to file '" + file + "'" );
        }
        finally{
            closeFileForOutput();
        }
    }

    public void clearRepository(){
        renewList();
    }

    public String toString(){
        if( items.isEmpty() ) return new String( "" );

        StringBuffer buffer = new StringBuffer();

        Iterator iter = items.iterator();
        int number = 0;
        while( iter.hasNext() ){
            buffer.append( "\nItem No: " + ++number + ":\n" );
            buffer.append( iter.next() );
        }

        return buffer.toString();
    }

    public int size(){
        return items.size();
    }


	//// Protected methods
	//
    protected Repository(){
        renewList();
    }

    protected Repository( Repository repo ) throws ZooException{
        items = (LinkedList<Serializable>) repo.getItems().clone();
    }


    //// Private methods
    //
    private void openFileForInput( String file ) throws ZooException{
        try{
            input = new ObjectInputStream( new FileInputStream( file ) );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while opening file '" + file + "' for input" );
        }
    }

    private void openFileForOutput( String file ) throws ZooException{
        try{
            output = new ObjectOutputStream( new FileOutputStream( file ) );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while opening file '" + file + "' for output" ) ;
        }
    }

    private void closeFileForInput() throws ZooException{
        try {
            if( input != null ){
                input.close();
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while closing file for input" );
        }
    }

    private void closeFileForOutput() throws ZooException{
        try {
            if( output != null ){
                output.flush();
                output.close();
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while closing file for output" );
        }
    }

    private void renewList(){
        items = new LinkedList<Serializable>();
    }

}
